from fastapi import APIRouter, Request, Depends
from motor.motor_asyncio import AsyncIOMotorDatabase

import schemas
from api.api_v1.check_data import service
from db import get_database

router = APIRouter()


@router.post("/check")
async def check(request: Request,
                data_in: schemas.CheckData,
                game: str,
                ) -> schemas.Msg:
    res = await service.check_data(game, data_in)
    return schemas.Msg(code=0, msg='ok', data=res)


@router.post("/save")
async def save(request: Request,
               data_in: schemas.AddTemplate,
               game: str,
               db: AsyncIOMotorDatabase = Depends(get_database),
               ) -> schemas.Msg:
    res = await service.save_template(db, data_in, game)
    return schemas.Msg(code=0, msg='ok', data=res)


@router.get('/template')
async def template(request: Request, game: str,
                   db: AsyncIOMotorDatabase = Depends(get_database),

                   ) -> schemas.Msg:
    data = await service.get_template(db, game)
    return schemas.Msg(code=0, msg='ok', data=data)


@router.post('/del_template')
async def del_template(request: Request, game: str, data_in: schemas.DelTemplate,
                       db: AsyncIOMotorDatabase = Depends(get_database),

                       ) -> schemas.Msg:
    data = await service.del_template(db, data_in)
    return schemas.Msg(code=0, msg='ok', data=data)


@router.get('/default_field')
async def template(request: Request, game: str) -> schemas.Msg:
    data = service.get_default_field()
    return schemas.Msg(code=0, msg='ok', data=data)
