import pymongo
from motor.motor_asyncio import AsyncIOMotorDatabase

from crud.base import CRUDBase
from schemas import *

__all__ = 'folder',


class CRUDFolder(CRUDBase):

    async def create(self, db: AsyncIOMotorDatabase, obj_in: FolderCreate, user_id: str):
        db_obj = FolderDB(
            **obj_in.dict(), user_id=user_id,
            members=[user_id],
            _id=uuid.uuid1().hex

        )
        await db[self.coll_name].insert_one(db_obj.dict(by_alias=True))

    async def read_folder(self, db, user_id, project_id, cat):
        return await self.read_have(db, user_id, project_id=project_id, cat=cat)

    async def create_index(self, db: AsyncIOMotorDatabase):
        await db[self.coll_name].create_index(
            [('project_id', pymongo.DESCENDING), ('name', pymongo.DESCENDING), ('user_id', pymongo.DESCENDING)],
            unique=True)


folder = CRUDFolder('folder')
