from enum import Enum
from typing import List

from pydantic import BaseModel


class AddRoleForUserInDomain(BaseModel):
    username: str
    role_id: str
    game: str
    auth_id: str


class AddRoleForUsersInDomain(BaseModel):
    data: List[AddRoleForUserInDomain]


class GetPermissionsForUserInDomain(BaseModel):
    role_id: str
    game: str


class DeleteRolesForUserInDomain(BaseModel):
    username: str
    role_id: str
    game: str


class Policy(BaseModel):
    role_id: str
    game: str
    path: str
    act: str = '*'


class AddPolicy(BaseModel):
    path_list: List[str]
    role_id: str
    game: str
    act: str = '*'


class DelPolicy(Policy):
    pass


class Ptype(str, Enum):
    p = 'p'
    g = 'g'


class CasbinRoleCreate(BaseModel):
    role_name: str
    role_api: List[str]


class CasbinDB(BaseModel):
    ptype: Ptype
    v0: str
    v1: str
    v2: str


class AccountCreate(BaseModel):
    username: str
    role_name: str
    # nickname: str
    data_auth_id: str


class AccountsCreate(BaseModel):
    accounts: List[AccountCreate]
    project_id: str


class AccountDeleteUser(BaseModel):
    name: str


class AccountSetRole(BaseModel):
    name: str
    role_name: str
